#include "enemies.h"
#include "map.h"

#include <GL/gl.h>
#include <allegro.h>
#include <cmath>
#include <cstdlib>

static const float GiantScale = 0.35;
static const float FallMaxSpeed = 1.0;
static const float GravityAccel = 0.01;
static const float HopSpeed = 0.2;
static const float HopHeight = 0.2;
static const int HopDelay = 200; // 4 seconds

Giant::Giant()
{
	radius = 1.5;
	height = 4.0;
	moveTimer = 0;
	collidable = true;
	deadly = true;
}

void Giant::render()
{
	const Model &model = resourceManager.getModel(ResourceManager::Giant);
	glDisable(GL_BLEND);
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, 
			resourceManager.getTexture(ResourceManager::GiantTexture));
	glColor3f(1.0, 1.0,1.0);
	
	glPushMatrix();
	glTranslatef(location.x, location.y, location.z);
	glRotatef(90, 0,1,0);
	glScalef( GiantScale, GiantScale, GiantScale);
	
	model.draw(true);	
	glPopMatrix();
}

void Giant::hop()
{
	if (!onGround) {
		return;
	}
	
	if (manager->trackObject == 0) {
		return;	
	}
	// Find out how far away (x) player is.
	float distx = manager->trackObject->location.x - location.x;
	if (std::fabs(distx) < (10 * TileSize)) {
		// hmm, hop.
		if (distx < 0) {
			velocity.x = - HopSpeed;
		} else {
			velocity.x = HopSpeed;
		}
		velocity.y = HopHeight;
		// 1 in 4 chance of hopping the wrong way.
		if ((rand() % 4) == 0) {
			velocity.x = - velocity.x;
		}
		moveTimer = HopDelay;
		manager->triggerSound(ResourceManager::Stomp, location);
	}
}

void Giant::tick()
{
	// Sort out velocity
	processFriction();
	// Apply gravity. 
	velocity = velocity + Vect3d(0, - GravityAccel, 0);
	if (velocity.y < (-FallMaxSpeed)) {
		velocity.y = -FallMaxSpeed;
	}
	if (moveTimer == 0) {
		hop();
	} else {
		moveTimer --;
	}
	location = location + velocity;
	float oldYVel = velocity.y;
	clip();
	if ((oldYVel < (- GravityAccel * 3)) && (velocity.y == 0)) {
		// Make a noise
		manager->triggerSound(ResourceManager::Stomp, location);
	}
}

