
#include <vector>
#include <string>

#include "resourcemanager.h"

class GameManager;

const float TileDepth = 6.0;
const float TileSize = 2.0;

enum TileType { Blank = 0, Door1 = 1, Door2 = 2, Spikes = 3, Wall = 4, Earth = 5 };

struct Tile {
	TileType type;
	bool indoors;
	bool climbable;
	bool earthUsed;
	// SOme more methods or properties?
	Tile() : type (Blank), indoors(false), climbable(false), earthUsed(false) {	} 
	
	bool isSolid() {
		return (type >= Wall);
	}
};

class Map {
	public:
		static const int MapWidth = 200;
		static const int MapHeight = 100;
	private:
		typedef std::vector <Tile> MapRow;
		typedef std::vector <MapRow> MapData;
		MapData data;
		void renderBlock(int x, int y);
		void renderWall(int x, int y);
		void renderObject(int x, int y, ResourceManager::ModelType modelType, double scale);
		void clear();
	public:
		Map();
		void render(int minx, int miny, int maxx, int maxy);
		void load(const std::string &filename, GameManager * man);
		Tile & tileAt(int x, int y) {
			return data[y][x];
		}
		Tile & tileAtLocation(double x, double y) {
			return data[(int) (y / TileSize)][(int) (x / TileSize)];
		}
		
		bool isSolid(double x, double y) {
			return tileAtLocation(x,y).isSolid();
		}
		bool isSpiky(double x, double y) {
			return tileAtLocation(x,y).type == Spikes;
		}
		bool isEarth(double x, double y) {
			Tile &t = tileAtLocation(x,y);
			return (t.type == Earth) && ! (t.earthUsed);
		}
		bool isClimbable(double x, double y) {
			return tileAtLocation(x,y).climbable;
		}
};

extern Map theMap;

