#include "stalk.h"

#include "resourcemanager.h"
#include "map.h"

#include <GL/gl.h>
#include <cmath>

static const float GrowthStages = 3.0;
static const float GrowthSpeed = 0.0161;
static const float StalkScale = (2.0 / 11.0);
static const float MaxHeight = 6.5;

Stalk::Stalk()
{
	height = 0; // initially.
	growth = 0;
}

void Stalk::render()
{
	int segments = (int) growth;
	float segmentGrowth = std::fmod(growth, 1.0);

	// Draw complete segments...
	glBindTexture(GL_TEXTURE_2D, 
		resourceManager.getTexture(ResourceManager::StalkTexture));
	const Model & completeModel = resourceManager.getModel(ResourceManager::Stalk3);
	for (int i=0; i<segments; i++) {
		float y = location.y + (i * TileSize);
		glPushMatrix();
		glTranslatef(location.x, y, location.z);
		glScalef( StalkScale, StalkScale, StalkScale);
		completeModel.draw(true);
		glPopMatrix();	
	}
	// Draw incomplete segment.
	if (segmentGrowth > 0.001) {
		float y = location.y + (segments * TileSize);
		int stage = (int) (segmentGrowth * GrowthStages);
		ResourceManager::ModelType fromType = 
			(ResourceManager::ModelType) (ResourceManager::Empty + stage);
		ResourceManager::ModelType toType = 
			(ResourceManager::ModelType) (fromType + 1);
		const Model &from = resourceManager.getModel(fromType);
		const Model &to = resourceManager.getModel(toType);
		float anim = std::fmod(segmentGrowth * GrowthStages, 1);
		glPushMatrix();
		glTranslatef(location.x, y, location.z);
		glScalef( StalkScale, StalkScale, StalkScale);
		from.drawTweened(true, to, anim);
		glPopMatrix();
	}
}

void Stalk::tick()
{
	if (growth < MaxHeight) {
		growth += GrowthSpeed;
		int segments = (int) growth;
		if (segments > 0) {
			int mapx = (int) (location.x / TileSize);
			int mapy = (int) ((location.y - 0.1) / TileSize);
			mapy = mapy + segments;
			theMap.tileAt(mapx, mapy).climbable = true;
		}
	}
}
