#include "utils.h"
#include "constants.h"
#include "resourcemanager.h"

#include <allegro.h>
#include <alleggl.h>

static void ClearIntermission()
{
	glClearColor(0,0,0.4, 0);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
  	glLoadIdentity();
	glOrtho(0, ScreenWidth, 0, ScreenHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glDisable(GL_BLEND);
}

void ShowIntermission(const std::string &caption)
{
	BITMAP *tempBitmap = create_bitmap(256, 16);
	clear_to_color(tempBitmap,0);
	int white = makecol(255,255,255);
	int black = 0;
	textout_ex(tempBitmap, font, caption.c_str(),1,1, white, black);
	GLuint tex = allegro_gl_make_texture_ex(
		AGL_TEXTURE_FLIP,
		tempBitmap,
		GL_LUMINANCE8);
	glBindTexture(GL_TEXTURE_2D, tex);
	destroy_bitmap(tempBitmap);
	
	// Intermission loop
	int counter = 200;
	while (counter > 0) {
		ClearIntermission();
		glEnable(GL_BLEND);
		glDisable(GL_LIGHTING);
		glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
		glEnable(GL_TEXTURE_2D);
		glColor3f(1,1,1);
		float yoffset = 350;
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0,yoffset);
		glTexCoord2f(1,0);
		glVertex2f(512,yoffset);
		glTexCoord2f(1,1);
		glVertex2f(512,32 + yoffset);
		glTexCoord2f(0,1);
		glVertex2f(0,32 + yoffset);
		glEnd();
		// draw quad.
		counter --;
		allegro_gl_flip();
		rest(20);
		if (key[KEY_SPACE] && (counter < 160)) {
			counter = 0;
		}
		if (key[KEY_ESC]) {
			counter = 0;
		}
	}
	
	glDeleteTextures(1, &tex);	
	rest(200); // small delay before we start the game.
}

// Must be an even number.
static char * titleCaptions[] = {
	"Space to start",
	"Escape to quit",
	"(c) 2005 Mark Robson",
	"Made in 48 hours",
	"Tools used:",
	"Bitmaps, textures: Gimp",
	"Models: Kludge3d",
	"Sounds: Audacity",
	"Editors: Kate, PFE32, vi",
	"Others: Perl",
	"Greetings to Buzz ",
	"for beta-testing.",
	"",
	"",
	0
};

// Show title screen - return true to play and false to quit.
bool ShowTitle()
{
	BITMAP *tempBitmap = create_bitmap(256, 32);
	int white = makecol(255,255,255);
	int black = 0;
	clear_to_color(tempBitmap,black);
	
	ClearIntermission();
	bool finished = false;
	bool startgame = false;
	float brightness = 0.0;
	int counter = 0;
	int msgCounter = 0;
	GLuint tex = 0;
	while (! finished)
	{
		if (brightness < 1.0) {
			brightness += 0.01;
		}
		ClearIntermission();
		glBindTexture(GL_TEXTURE_2D, 
			resourceManager.getTexture(ResourceManager::TitleTexture));
		glEnable(GL_TEXTURE_2D);
		glColor3f(brightness, brightness, brightness);
		float texture_height = (3.0 / 4.0);
		float texture_miny = (1.0 - texture_height) /2.0;
		float texture_maxy = 1.0 - texture_miny;
		glBegin(GL_QUADS);
		glTexCoord2f(0,texture_miny);
		glVertex2f(0,0);
		glTexCoord2f(1,texture_miny);
		glVertex2f(ScreenWidth,0);
		glTexCoord2f(1,texture_maxy);
		glVertex2f(ScreenWidth,ScreenHeight);
		glTexCoord2f(0,texture_maxy);
		glVertex2f(0,ScreenHeight);
		glEnd();
		// display next message
		if (counter == 0) {
			clear_to_color(tempBitmap,0);
			textout_ex(tempBitmap, font, titleCaptions[msgCounter],1,1, white, black);
			textout_ex(tempBitmap, font, titleCaptions[msgCounter+1],1,10, white, black);
			if (tex != 0) {
				glDeleteTextures(1, &tex);
			}

			tex = allegro_gl_make_texture_ex(
				AGL_TEXTURE_FLIP,
				tempBitmap,
				GL_LUMINANCE8);
			counter = 150;
			msgCounter += 2;
			if (titleCaptions[msgCounter] == 0) {
				msgCounter = 0;
			}
		} else {
			counter --;
		}
		
		float textbright = 1.0;
		if (counter <50) {
			textbright = ((float) counter) / 50;
		}
		if (counter > 100) {
			textbright = ((float) (150 - counter)) / 50;
		}
		glColor3f(textbright, textbright, textbright);
		glBindTexture(GL_TEXTURE_2D, tex);
		glEnable(GL_BLEND);
		glDisable(GL_LIGHTING);
		glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);
		glEnable(GL_TEXTURE_2D);
		float yoffset = 100;
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(0,yoffset);
		glTexCoord2f(1,0);
		glVertex2f(512,yoffset);
		glTexCoord2f(1,1);
		glVertex2f(512,64 + yoffset);
		glTexCoord2f(0,1);
		glVertex2f(0,64 + yoffset);
		glEnd();
				
		allegro_gl_flip();
		rest(20);
		if (key[KEY_ESC]) {
			finished = true;
		}
		if (key[KEY_SPACE]) {
			finished = true;
			startgame = true;
		}
	}
	if (tex != 0) {
		glDeleteTextures(1, &tex);
	}

	destroy_bitmap(tempBitmap);
	return startgame;
}




